@@ -13,8 +13,8 @@ use Perinci::Object;
 use Perinci::ToUtil;
 use Scalar::Util qw(reftype blessed);
 
-our $VERSION = '1.15'; # VERSION
-our $DATE = '2014-07-02'; # DATE
+our $VERSION = '1.16'; # VERSION
+our $DATE = '2014-07-03'; # DATE
 
 our $REQ_VERSION = 0; # version requested by user
 
@@ -726,7 +726,15 @@ sub run_completion {
     }
 
   DISPLAY_RES:
-    print Complete::Bash::format_completion($res);
+    my $compres = Complete::Bash::format_completion($res);
+    if ($ENV{PERINCI_CMDLINE_SERVER}) {
+        # a flag that causes completion result to be stored in the object
+        # instead of printed. this is because we want to return result in a
+        # function
+        $self->{_compres} = $compres;
+    } else {
+        print $compres;
+    }
     0;
 }
 
@@ -1816,12 +1824,43 @@ sub _load_log_any_app {
     Log::Any::App::init();
 
     # we log this after we initialize Log::Any::App, since Log::Any::App might
-    # not be loaded at all. yes, this means that this log message is printer
+    # not be loaded at all. yes, this means that this log message is printed
     # rather late and might not be the first message to be logged (see log
     # messages in run()) if user already loads Log::Any::App by herself.
-    $self->{_original_argv} =
-        $log->debugf("Program %s started with arguments: %s",
-                     $0, $self->{_orig_argv});
+    $log->debugf("Program %s started with arguments: %s",
+                 $0, $self->{_orig_argv});
+}
+
+# since there's now module like Perinci::CmdLine::Server which might reuse the
+# same instance for multiple run()'s/requests, use this to clean state between
+# requests.
+sub _init_request {
+    my ($self) = @_;
+    $self->{_actions} = []; # first action will be tried first, then 2nd, ...
+    undef $self->{_selected_subcommand};
+    undef $self->{_force_call};
+    undef $self->{_check_required_args};
+    undef $self->{_subcommand};
+    undef $self->{_args};
+    undef $self->{_send_argv};
+    undef $self->{_orig_argv};
+    undef $self->{_getargs_result};
+    undef $self->{_comp_parse_res};
+    undef $self->{_go_specs_common};
+    undef $self->{_scn_in_argv};
+    undef $self->{_dry_run};
+    undef $self->{_help_curtbl};
+    undef $self->{_help_info};
+    undef $self->{_help_meta};
+    undef $self->{_some_subcommands_not_shown_in_help};
+    undef $self->{_tx_id};
+    undef $self->{_meta};
+    undef $self->{_res};
+    undef $self->{_fres};
+    undef $self->{_compres};
+
+    # not reset
+    #$self->{_log_any_app_loaded};
 }
 
 sub run {
@@ -1829,6 +1868,8 @@ sub run {
 
     $log->trace("-> CmdLine's run()");
 
+    $self->_init_request;
+
     #
     # workaround: detect (1) if we're being invoked for bash completion, get
     # @ARGV from parsing COMP_LINE/COMP_POINT instead, since @ARGV given by bash
@@ -1842,8 +1883,6 @@ sub run {
         $self->{_comp_parse_res} = [$words, $cword]; # store for run_completion()
     }
 
-    $self->{_actions} = []; # first action will be tried first, then 2nd, ...
-
     #
     # parse common opts first so we can catch --help, --subcommands, etc.
     #
@@ -1926,7 +1965,7 @@ Perinci::CmdLine - Rinci/Riap-based command-line application framework
 
 =head1 VERSION
 
-This document describes version 1.15 of Perinci::CmdLine (from Perl distribution Perinci-CmdLine), released on 2014-07-02.
+This document describes version 1.16 of Perinci::CmdLine (from Perl distribution Perinci-CmdLine), released on 2014-07-03.
 
 =head1 SYNOPSIS
 
